<?php

namespace Drupal\pagetree\Service;

use Drupal\system\Entity\Menu;
use Drupal\node\Entity\Node;
use Drupal\Core\Cache\CacheableMetadata;

/**
 * Provides a resource to get the page tree.
 */
class Tree {
  protected $_contentTypes = [];
  protected $_languagesUsed = [];
  protected $_nids = [];
  protected $_permissions = [];
  protected $_handlers = [];
  protected $handledIds = [];
  protected $cache = NULL;

  /**
   * Create a new tree service.
   */
  public function __construct() {
    $this->_contentTypes = \Drupal::configFactory()->get('pagetree.settings')->get('contentTypes');
    $languagesSelected = \Drupal::configFactory()->get('pagetree.settings')->get('languages');
    $this->_languagesUsed = [];
    $translations = [];
    $languages = \Drupal::service('language_manager')->getLanguages();
    $this->_languagesUsed = [];
    if ($languagesSelected != NULL && count($languagesSelected) > 0) {
      foreach ($languages as $language) {
        if (in_array($language->getId(), $languagesSelected)) {
          $this->_languagesUsed[] = $language;
        }
      }
    }
    else {
      $this->_languagesUsed = $languages;
    }
    $this->_handlers = \Drupal::service('plugin.manager.pagetree_state_handler_manager')->getHandlers();
    $this->cache = new CacheableMetadata();
  }

  /**
   * Request the current page tree.
   *
   * @return array
   *   The prepared array tree
   */
  public function get() {
    // Get nids in the menus selected in the configuration.
    $menus = \Drupal::configFactory()->get('pagetree.settings')->get('menus');
    $menuHelper = \Drupal::service('frontendpublishing.menu_helper');
    $nids = [];
    $menuTrees = [];
    foreach ($menus as $menuId) {
      $menuTrees[$menuId] = $menuHelper::getMenuTree($menuId);
      $this->collectNids($menuTrees[$menuId], $nids);
    }
    if (count($nids) == 0) {
      return [];
    }

    // Get node and revision information, build the entity information array.
    $nodes = $this->getNodes($nids);
    $revisions = $this->getLatestRevisions($nids);
    foreach ($this->_handlers as $handler) {
      $handler->annotate($revisions);
    }
    $entityInfos = [];
    $this->processNodes($nodes, $revisions, $entityInfos);

    // Generate array trees for frontend display.
    $trees = [];
    foreach ($menus as $menuId) {
      $menu = Menu::load($menuId);
      if ($menu != NULL) {
        // Combine the info.
        $arrayTree = $this->buildArrayTree($menuTrees[$menuId], $entityInfos);
        $trees[] = ['label' => $menu->label(), 'id' => $menu->id(), 'langcode' => $menu->language()->getId(), 'tree' => $arrayTree];
      }
    }
    $this->cache->addCacheableDependency(
      CacheableMetadata::createFromRenderArray(
        [
          '#cache' =>
            [
              'tags' => [
                'menu_link_content_list',
              ],
              'contexts' => [
                'user',
              ],
            ],
        ]
      )
    );
    $data = ['hash' => md5(serialize($trees)), 'trees' => $trees, '#cache' => $this->cache];
    return $data;
  }

  /**
   * Process the nodes to display.
   *
   * Use the node and revision information to collect all necessary information
   * for the pagetree.
   *
   * @param array $nodes
   *   The nodes to display.
   * @param array $revisions
   *   The revision information.
   * @param array $entityInfos
   *   The resulting entity information (inout).
   */
  protected function processNodes(array &$nodes, array &$revisions, array &$entityInfos) {

    foreach ($this->_languagesUsed as &$language) {
      if (!empty($nodes[$language->getId()])) {
        foreach ($nodes[$language->getId()] as $node) {
          $this->cache->addCacheableDependency($node);
          if (empty($entityInfos[$node->id()])) {
            $entityInfos[$node->id()] = ['bundle' => $node->getType(), 'translations' => [], 'permissions' => $this->getPermissions($node)];
          }
          if (!empty($revisions[$node->id() . $language->getId()])) {
            $revision = $revisions[$node->id() . $language->getId()];
            if (!empty($revision['#cache'])) {
              $this->cache->addCacheableDependency(CacheableMetadata::createFromRenderArray($revision));
            }
            $status = 0;
            $translation = [];
            if ($node->status->value == 1) {
              if ($revision['status'] == 1) {
                $status = 1;
              }
              else {
                $status = 2;
              }
            }
            $externalLink = \Drupal::service('path.alias_manager')->getAliasByPath('/node/' . $node->id(), $language->getId());
            $settingsLink = '/node/' . $node->id() . '/edit';
            if (\Drupal::languageManager()->isMultilingual()) {
              $externalLink = '/' . $language->getId() . $externalLink;
              $settingsLink = '/' . $language->getId() . $settingsLink;
            }

            $translation['name'] = $revision['title'];
            $translation['status'] = $status;
            $translation['externalLink'] = $externalLink;
            $translation['nodeId'] = $node->id();
            $translation['language'] = $language->getId();
            $translation['defaultTranslation'] = $node->getUntranslated()->language()->getId();
            $entityInfos[$node->id()]['translations'][$language->getId()] = $translation;
          }
          else {
            $entityInfos[$node->id()]['translations'][$language->getId()] = ['name' => 'No translation exists.', 'status' => -1];
          }
        }
      }
    }
    foreach ($entityInfos as $nodeId => $node) {
      foreach ($this->_languagesUsed as &$language) {
        if (empty($entityInfos[$nodeId]['translations'][$language->getId()])) {
          $entityInfos[$nodeId]['translations'][$language->getId()] = ['name' => 'No translation exists.', 'status' => -1];
        }
      }
    }
  }

  /**
   * Collect the node information based on the given node ids.
   *
   * @param array $nids
   *   The node ids to load the information for.
   *
   * @return array
   *   The node information.
   */
  public function getNodes(array $nids) {
    $nodes = Node::loadMultiple($nids);
    $return = [];
    foreach ($nodes as $node) {
      $languages = $node->getTranslationLanguages(TRUE);
      foreach ($languages as $language) {
        $return[$language->getId()][] = $node;
      }
    }
    return $return;
  }

  /**
   * Collect the revision information based on the given revision ids.
   *
   * @param array $rids
   *   The revision ids to load the information for.
   *
   * @return array
   *   The revision information.
   */
  public function getLatestRevisions(array $rids) {
    $revisions = [];
    foreach ($this->_languagesUsed as &$language) {
      $stmt = \Drupal::database()->query(
            "SELECT a.nid, a.langcode, a.status, a.vid, link.title FROM node_field_revision a JOIN (SELECT vid FROM (SELECT MAX(vid) as vid FROM node_field_revision WHERE `nid` IN (:nids[]) AND langcode LIKE :langcode AND revision_translation_affected = 1 GROUP BY changed, nid, langcode ORDER BY `changed` DESC) as sub) b on a.vid=b.vid AND a.langcode LIKE :langcode JOIN menu_link_content_data link ON link__uri = CONCAT('entity:node/', a.nid) OR link__uri = CONCAT('internal:/node/', a.nid) AND link.langcode = :langcode",
            [
              ':nids[]' => $rids,
              ':langcode' => $language->getId(),
            ]
        );
      $results = $stmt->fetchAll(\PDO::FETCH_OBJ);
      foreach ($results as $row) {
        if (empty($revisions[$row->nid . $row->langcode]) || $revisions[$row->nid . $row->langcode]['vid'] < $row->vid) {
          $revisions[$row->nid . $row->langcode] = ['nid' => $row->nid, 'langcode' => $row->langcode, 'title' => $row->title, 'status' => $row->status, 'vid' => $row->vid, '#cache' => []];
        }
      }
    }
    return $revisions;
  }

  /**
   * Create a array tree representation of the menu based on the given menutree.
   * Annotate it with info from entityInfos.
   *
   * @param \Drupal\system\Entity\MenuTree $tree
   *   The menu tree.
   * @param array[] $entityInfos
   *   The entity information.
   *
   * @return array
   *   The entity information as a tree.
   */
  public function buildArrayTree(&$tree, &$entityInfos) {
    $return = [];
    foreach ($tree as $entry) {
      if (array_key_exists('node', $entry->link->getRouteParameters())) {
        $id = $entry->link->getRouteParameters()['node'];
        if (isset($entityInfos[$id])) {
          $linkId = str_replace('menu_link_content:', '', $entry->link->getPluginId());
          $output = [
            'id' => $id,
            'bundle' => $entityInfos[$id]['bundle'],
            'linkId' => $linkId,
            'translations' => &$entityInfos[$id]['translations'],
            'permissions' => &$entityInfos[$id]['permissions'],
          ];

          if (!in_array($id, $this->handledIds)) {
            // Allow handlers to alter the entries.
            foreach ($this->_handlers as $handler) {
              $handler->alterEntry($output);
            }
            // Sort actions by weight.
            foreach ($output['translations'] as $language => &$translation) {
              if ($translation != NULL && !empty($translation['actions'])) {
                usort($translation['actions'], ['\Drupal\Component\Utility\SortArray', 'sortByWeightElement']);
              }
            }
            $this->handledIds[] = $id;
          }

          $output['children'] = $this->buildArrayTree($entry->subtree, $entityInfos);
          $return[] = $output;
        }
      }
    }
    return $return;
  }

  /**
   * Collect all the node ids from a menu tree.
   *
   * @param \Drupal\system\Entity\MenuTree $tree
   *   The tree to parse.
   * @param array $nids
   *   The node ids (inout);.
   *
   * @return array
   *   The node ids;
   */
  public function collectNids($tree, &$nids = []) {
    foreach ($tree as $entry) {
      if (array_key_exists('node', $entry->link->getRouteParameters())) {
        $nids[] = $entry->link->getRouteParameters()['node'];
        $this->collectNids($entry->subtree, $nids);
      }
    }
    return $nids;
  }

  /**
   * Get the permissions for the current user.
   *
   * @param string $bundle
   *   The bundle to collect the permissions for.
   *
   * @return array
   *   The permissions for the given bundle and the current user.
   */
  public function getPermissions($node) {
    $permissions['create'] = $node->access('create');
    $permissions['update'] = $node->access('update');
    $permissions['delete'] = $node->access('delete');
    return $permissions;
  }

}
